@@ -1,82 +1,60 @@
 # -*- coding: utf-8 -*-
 from __future__ import absolute_import, print_function, with_statement
+from typing import Iterable
+from typing import Union
+from typing import Generator
 
-import logging
 import os
 import re
 import subprocess
 import sys
 import tempfile
 import textwrap
-import warnings
+import glob
+from pathlib import Path
 
-from .handler import _check_log_handler
+from .handler import logger, _check_log_handler
 from .pandoc_download import DEFAULT_TARGET_FOLDER, download_pandoc
 from .py3compat import cast_bytes, cast_unicode, string_types, url2path, urlparse
 
 __author__ = u'Juho Vepsäläinen'
-__version__ = '1.7.4'
+__author_email__ = "bebraw@gmail.com"
+__maintainer__ = u'Jessica Tegner'
+__url__ = 'https://github.com/JessicaTegner/pypandoc'
+__version__ = '1.12'
 __license__ = 'MIT'
-__all__ = ['convert', 'convert_file', 'convert_text',
+__description__ = "Thin wrapper for pandoc."
+__python_requires__ = ">=3.6"
+__setup_requires__ = ['setuptools', 'pip>=8.1.0', 'wheel>=0.25.0']
+__classifiers__ = [
+        'Development Status :: 4 - Beta',
+        'Environment :: Console',
+        'Intended Audience :: Developers',
+        'Intended Audience :: System Administrators',
+        'License :: OSI Approved :: MIT License',
+        'Operating System :: POSIX',
+        'Programming Language :: Python',
+        'Topic :: Text Processing',
+        'Topic :: Text Processing :: Filters',
+        'Programming Language :: Python :: 3',
+        'Programming Language :: Python :: 3.6',
+        'Programming Language :: Python :: 3.7',
+        'Programming Language :: Python :: 3.8',
+        'Programming Language :: Python :: 3.9',
+        'Programming Language :: Python :: 3.10',
+        'Programming Language :: Python :: 3.11',
+        'Programming Language :: Python :: 3.12',
+        'Programming Language :: Python :: Implementation :: CPython',
+        'Programming Language :: Python :: Implementation :: PyPy'
+    ]
+
+__all__ = ['convert_file', 'convert_text',
            'get_pandoc_formats', 'get_pandoc_version', 'get_pandoc_path',
            'download_pandoc']
 
-# Set up the module level logger
-logger = logging.getLogger(__name__)
-
-
-def convert(source, to, format=None, extra_args=(), encoding='utf-8',
-            outputfile=None, filters=None, cworkdir=None):
-    """Converts given `source` from `format` to `to` (deprecated).
-
-    :param str source: Unicode string or bytes or a file path (see encoding)
-
-    :param str to: format into which the input should be converted; can be one of
-            `pypandoc.get_pandoc_formats()[1]`
-
-    :param str format: the format of the inputs; will be inferred if input is a file with an
-            known filename extension; can be one of `pypandoc.get_pandoc_formats()[1]`
-            (Default value = None)
-
-    :param list extra_args: extra arguments (list of strings) to be passed to pandoc
-            (Default value = ())
-
-    :param str encoding: the encoding of the file or the input bytes (Default value = 'utf-8')
-
-    :param str outputfile: output will be written to outfilename or the converted content
-            returned if None (Default value = None)
-
-    :param list filters: pandoc filters e.g. filters=['pandoc-citeproc']
-
-    :returns: converted string (unicode) or an empty string if an outputfile was given
-    :rtype: unicode
-
-    :raises RuntimeError: if any of the inputs are not valid of if pandoc fails with an error
-    :raises OSError: if pandoc is not found; make sure it has been installed and is available at
-            path.
-    """
-    msg = ("Due to possible ambiguity, 'convert()' is deprecated and will be removed in pypandoc 1.8. "
-           "Use 'convert_file()'  or 'convert_text()'.")
-    warnings.warn(msg, DeprecationWarning, stacklevel=2)
-
-    path = _identify_path(source)
-    if path:
-        format = _identify_format_from_path(source, format)
-        input_type = 'path'
-    else:
-        source = _as_unicode(source, encoding)
-        input_type = 'string'
-        if not format:
-            raise RuntimeError("Format missing, but need one (identified source as text as no "
-                               "file with that name was found).")
-    return _convert_input(source, format, input_type, to, extra_args=extra_args,
-                          outputfile=outputfile, filters=filters,
-                          cworkdir=cworkdir)
-
-
-def convert_text(source, to, format, extra_args=(), encoding='utf-8',
-                 outputfile=None, filters=None, verify_format=True,
-                 sandbox=True, cworkdir=None):
+def convert_text(source:str, to:str, format:str, extra_args:Iterable=(), encoding:str='utf-8',
+                 outputfile:Union[None, str, Path]=None, filters:Union[Iterable, None]=None, verify_format:bool=True,
+                 sandbox:bool=False, cworkdir:Union[str, None]=None) -> str:
     """Converts given `source` from `format` to `to`.
 
     :param str source: Unicode string or bytes (see encoding)
@@ -91,8 +69,9 @@ def convert_text(source, to, format, extra_args=(), encoding='utf-8',
 
     :param str encoding: the encoding of the input bytes (Default value = 'utf-8')
 
-    :param str outputfile: output will be written to outfilename or the converted content
-            returned if None (Default value = None)
+    :param str outputfile: output will be written to outputfile or the converted content
+            returned if None. The output filename can be specified as a string
+            or pathlib.Path object. (Default value = None)
 
     :param list filters: pandoc filters e.g. filters=['pandoc-citeproc']
 
@@ -100,7 +79,7 @@ def convert_text(source, to, format, extra_args=(), encoding='utf-8',
             (Default value = True)
 
     :param bool sandbox: Run pandoc in pandocs own sandbox mode, limiting IO operations in readers and writers to reading the files specified on the command line. Anyone using pandoc on untrusted user input should use this option. Note: This only does something, on pandoc >= 2.15
-            (Default value = True)
+            (Default value = False)
 
     :returns: converted string (unicode) or an empty string if an outputfile was given
     :rtype: unicode
@@ -116,12 +95,17 @@ def convert_text(source, to, format, extra_args=(), encoding='utf-8',
                           cworkdir=cworkdir)
 
 
-def convert_file(source_file, to, format=None, extra_args=(), encoding='utf-8',
-                 outputfile=None, filters=None, verify_format=True,
-                 sandbox=True, cworkdir=None):
+def convert_file(source_file:Union[list, str, Path, Generator], to:str, format:Union[str, None]=None,
+                 extra_args:Iterable=(), encoding:str='utf-8', outputfile:Union[None, str, Path]=None,
+                 filters:Union[Iterable, None]=None, verify_format:bool=True, sandbox:bool=False,
+                 cworkdir:Union[str, None]=None) -> str:
     """Converts given `source` from `format` to `to`.
 
-    :param str source_file: file path (see encoding)
+    :param (str, list, pathlib.Path) source_file: If a string, should be either
+            an absolute file path, relative file path, or a file pattern (like dir/*.md).
+            If a list, should be a list of file paths, file patterns, or pathlib.Path
+            objects. In addition, pathlib.Path objects as well as the generators produced by
+            pathlib.Path.glob may be specified.
 
     :param str to: format into which the input should be converted; can be one of
             `pypandoc.get_pandoc_formats()[1]`
@@ -133,10 +117,11 @@ def convert_file(source_file, to, format=None, extra_args=(), encoding='utf-8',
     :param list extra_args: extra arguments (list of strings) to be passed to pandoc
             (Default value = ())
 
-    :param str encoding: the encoding of the file or the input bytes (Default value = 'utf-8')
+    :param str encoding (deprecated): the encoding of the input bytes (Default value = 'utf-8')
 
-    :param str outputfile: output will be written to outfilename or the converted content
-            returned if None (Default value = None)
+    :param str outputfile: output will be written to outputfile or the converted content
+            returned if None. The output filename can be specified as a string
+            or pathlib.Path object. (Default value = None)
 
     :param list filters: pandoc filters e.g. filters=['pandoc-citeproc']
 
@@ -144,7 +129,7 @@ def convert_file(source_file, to, format=None, extra_args=(), encoding='utf-8',
             (Default value = True)
 
     :param bool sandbox: Run pandoc in pandocs own sandbox mode, limiting IO operations in readers and writers to reading the files specified on the command line. Anyone using pandoc on untrusted user input should use this option. Note: This only does something, on pandoc >= 2.15
-            (Default value = True)
+            (Default value = False)
 
     :returns: converted string (unicode) or an empty string if an outputfile was given
     :rtype: unicode
@@ -153,20 +138,50 @@ def convert_file(source_file, to, format=None, extra_args=(), encoding='utf-8',
     :raises OSError: if pandoc is not found; make sure it has been installed and is available at
             path.
     """
+    # TODO: remove 'encoding' parameter and warning
+    if encoding != "utf-8":
+        logger.warning("The 'encoding' parameter will be removed in version 1.13. Just remove the parameter, because currently the method does not use it.")
+
+    # This if block effectively adds support for pathlib.Path objects
+    # and generators produced by pathlib.Path().glob().
+    if not isinstance(source_file, str):
+        try:
+            source_file = list(map(str, source_file))
+        except TypeError:
+            source_file = str(source_file)
+
     if not _identify_path(source_file):
         raise RuntimeError("source_file is not a valid path")
-    format = _identify_format_from_path(source_file, format)
-    return _convert_input(source_file, format, 'path', to, extra_args=extra_args,
+    if _is_network_path(source_file): # if the source_file is an url
+        format = _identify_format_from_path(source_file, format)
+        return _convert_input(source_file, format, 'path', to, extra_args=extra_args,
                           outputfile=outputfile, filters=filters,
                           verify_format=verify_format, sandbox=sandbox,
                           cworkdir=cworkdir)
 
+    discovered_source_files = []
+    if isinstance(source_file, str):
+        discovered_source_files += glob.glob(source_file)
+    if isinstance(source_file, list): # a list of possibly file or file patterns. Expand all with glob
+        for filepath in source_file:
+            discovered_source_files.extend(glob.glob(filepath))
 
-def _identify_path(source):
-    # guard against problems
-    if source is None or not isinstance(source, string_types):
-        return False
+    format = _identify_format_from_path(discovered_source_files[0], format)
+    if len(discovered_source_files) == 1:
+        discovered_source_files = discovered_source_files[0]
 
+    return _convert_input(discovered_source_files, format, 'path', to, extra_args=extra_args,
+                      outputfile=outputfile, filters=filters,
+                      verify_format=verify_format, sandbox=sandbox,
+                      cworkdir=cworkdir)
+
+
+def _identify_path(source) -> bool:
+    if isinstance(source, list):
+        for single_source in source:
+            if not _identify_path(single_source):
+                return False
+        return True
     is_path = False
     try:
         is_path = os.path.exists(source)
@@ -177,24 +192,51 @@ def _identify_path(source):
         pass
 
     if not is_path:
-        # check if it's an URL
-        result = urlparse(source)
-        if result.scheme in ["http", "https"]:
-            is_path = True
-        elif result.scheme and result.netloc and result.path:
-            # complete uri including one with a network path
-            is_path = True
-        elif result.scheme == "file" and result.path:
-            is_path = os.path.exists(url2path(source))
+        try:
+            is_path = len(glob.glob(source)) >= 1
+        except UnicodeEncodeError:
+            is_path = len(glob.glob(source.encode('utf-8'))) >= 1
+        except:  # noqa
+            # still false
+            pass
+    
+    if not is_path:
+        try:
+            # check if it's an URL
+            result = urlparse(source)
+            if result.scheme in ["http", "https"]:
+                is_path = True
+            elif result.scheme and result.netloc and result.path:
+                # complete uri including one with a network path
+                is_path = True
+            elif result.scheme == "file" and result.path:
+                is_path = os.path.exists(url2path(source))
+        except AttributeError:
+            pass
 
     return is_path
 
+def _is_network_path(source):
+    try:
+        # check if it's an URL
+        result = urlparse(source)
+        if result.scheme in ["http", "https"]:
+            return True
+        elif result.scheme and result.netloc and result.path:
+            # complete uri including one with a network path
+            return True
+        elif result.scheme == "file" and result.path:
+            return os.path.exists(url2path(source))
+    except AttributeError:
+        pass
+    return False
 
-def _identify_format_from_path(sourcefile, format):
+
+def _identify_format_from_path(sourcefile:str, format:str) -> str:
     return format or os.path.splitext(sourcefile)[1].strip('.')
 
 
-def _as_unicode(source, encoding):
+def _as_unicode(source:any, encoding:str) -> any:
     if encoding != 'utf-8':
         # if a source and a different encoding is given, try to decode the the source into a
         # unicode string
@@ -205,7 +247,7 @@ def _as_unicode(source, encoding):
     return source
 
 
-def _identify_input_type(source, format, encoding='utf-8'):
+def _identify_input_type(source:any, format:str, encoding:str='utf-8'):
     path = _identify_path(source)
     if path:
         format = _identify_format_from_path(source, format)
@@ -266,8 +308,16 @@ def _validate_formats(format, to, outputfile):
     if base_to_format == "pdf":
         # pdf formats needs to actually have a to format of latex and a
         # filename with an ending pf .pdf
-        if outputfile[-4:] != ".pdf":
-            raise RuntimeError('PDF output needs an outputfile with ".pdf" as a fileending.')
+        if isinstance(outputfile, str):
+            if outputfile[-4:] != ".pdf":
+                raise RuntimeError(
+                    'PDF output needs an outputfile with ".pdf" as a fileending.'
+                )
+        elif isinstance(outputfile, Path):
+            if outputfile.suffix != ".pdf":
+                raise RuntimeError(
+                    'PDF output needs an outputfile with ".pdf" as a fileending.'
+                )
         # to is not allowed to contain pdf, but must point to latex
         # it's also not allowed to contain extensions according to the docs
         if to != base_to_format:
@@ -279,19 +329,31 @@ def _validate_formats(format, to, outputfile):
 
 def _convert_input(source, format, input_type, to, extra_args=(),
                    outputfile=None, filters=None, verify_format=True,
-                   sandbox=True, cworkdir=None):
+                   sandbox=False, cworkdir=None):
     
     _check_log_handler()
+
+    logger.debug("Ensuring pandoc path...")
     _ensure_pandoc_path()
 
     if verify_format:
+        logger.debug("Verifying format...")
         format, to = _validate_formats(format, to, outputfile)
     else:
         format = normalize_format(format)
         to = normalize_format(to)
 
+    logger.debug("Identifying input type...")
     string_input = input_type == 'string'
-    input_file = [source] if not string_input else []
+    if not string_input:
+        if isinstance(source, str):
+            input_file = [source]
+        else:
+            input_file = source
+    else:
+        input_file = []
+    
+    input_file = sorted(input_file)
     args = [__pandoc_path, '--from=' + format]
 
     args.append('--to=' + to)
@@ -299,11 +361,14 @@ def _convert_input(source, format, input_type, to, extra_args=(),
     args += input_file
 
     if outputfile:
-        args.append("--output=" + outputfile)
+        args.append("--output=" + str(outputfile))
 
     if sandbox:
         if ensure_pandoc_minimal_version(2,15): # sandbox was introduced in pandoc 2.15, so only add if we are using 2.15 or above.
+            logger.debug("Adding sandbox argument...")
             args.append("--sandbox")
+        else:
+            logger.warning("Sandbox argument was used, but pandoc version is too low. Ignoring argument.")
 
     args.extend(extra_args)
 
@@ -311,7 +376,7 @@ def _convert_input(source, format, input_type, to, extra_args=(),
     if filters is not None:
         if isinstance(filters, string_types):
             filters = filters.split()
-        f = ['--filter=' + x for x in filters]
+        f = ['--lua-filter=' + x if x.endswith(".lua") else '--filter=' + x for x in filters]
         args.extend(f)
 
     # To get access to pandoc-citeproc when we use a included copy of pandoc,
@@ -325,6 +390,7 @@ def _convert_input(source, format, input_type, to, extra_args=(),
     if cworkdir and old_wd != cworkdir:
         os.chdir(cworkdir)
 
+    logger.debug("Running pandoc...")
     p = subprocess.Popen(
         args,
         stdin=subprocess.PIPE if string_input else None,
@@ -343,11 +409,12 @@ def _convert_input(source, format, input_type, to, extra_args=(),
                                                                            p.stderr.read())
         )
 
-    try:
-        source = cast_bytes(source, encoding='utf-8')
-    except (UnicodeDecodeError, UnicodeEncodeError):
-        # assume that it is already a utf-8 encoded string
-        pass
+    if string_input:
+        try:
+            source = cast_bytes(source, encoding='utf-8')
+        except (UnicodeDecodeError, UnicodeEncodeError):
+            # assume that it is already a utf-8 encoded string
+            pass
     try:
         stdout, stderr = p.communicate(source if string_input else None)
     except OSError:
@@ -383,16 +450,27 @@ def _convert_input(source, format, input_type, to, extra_args=(),
 
 
 def _classify_pandoc_logging(raw, default_level="WARNING"):
-    # Process raw and yeild the contained logging levels and messages.
+    # Process raw and yield the contained logging levels and messages.
     # Assumes that the messages are formatted like "[LEVEL] message". If the 
-    # first message does not have a level, use the default_level value instead.
+    # first message does not have a level or any other message has a level 
+    # that does not conform to the pandoc standard, use the default_level 
+    # value instead.
     
-    level_map = {"CRITICAL": 50,
-                 "ERROR": 40,
-                 "WARNING": 30,
-                 "INFO": 20,
-                 "DEBUG": 10,
-                 "NOTSET": 0}
+    # Available pandoc logging levels adapted from:
+    # https://github.com/jgm/pandoc/blob/5e1249481b2e3fc27e845245a0c96c3687a23c3d/src/Text/Pandoc/Logging.hs#L44
+    def get_python_level(pandoc_level):
+        
+        level_map = {"ERROR": 40,
+                     "WARNING": 30,
+                     "INFO": 20,
+                     "DEBUG": 10}
+        
+        if pandoc_level not in level_map:
+            level = level_map[default_level]
+        else:
+            level = level_map[pandoc_level]
+        
+        return level
     
     msgs = raw.split("\n")
     first = msgs.pop(0)
@@ -401,25 +479,25 @@ def _classify_pandoc_logging(raw, default_level="WARNING"):
     
     # Use the default if the first message doesn't have a level
     if search is None:
-        level = default_level
+        pandoc_level = default_level
     else:
-        level = first[search.start(1):search.end(1)]
+        pandoc_level = first[search.start(1):search.end(1)]
     
-    log_msgs = [first.replace('[{}] '.format(level), '')]
+    log_msgs = [first.replace('[{}] '.format(pandoc_level), '')]
     
     for msg in msgs:
         
         search = re.search(r"\[(.*?)\]", msg)
         
         if search is not None:
-            yield level_map[level], "\n".join(log_msgs)
-            level = msg[search.start(1):search.end(1)]
-            log_msgs = [msg.replace('[{}] '.format(level), '')]
+            yield get_python_level(pandoc_level), "\n".join(log_msgs)
+            pandoc_level = msg[search.start(1):search.end(1)]
+            log_msgs = [msg.replace('[{}] '.format(pandoc_level), '')]
             continue
         
         log_msgs.append(msg)
     
-    yield level_map[level], "\n".join(log_msgs)
+    yield get_python_level(pandoc_level), "\n".join(log_msgs)
 
 
 def _get_base_format(format):
@@ -432,7 +510,7 @@ def _get_base_format(format):
     return re.split(r'\+|-', format)[0]
 
 
-def get_pandoc_formats():
+def get_pandoc_formats() -> Iterable:
     '''
     Dynamic preprocessor for Pandoc formats.
     Return 2 lists. "from_formats" and "to_formats".
@@ -465,7 +543,7 @@ def get_pandoc_formats():
     return [f.strip() for f in in_], [f.strip() for f in out]
 
 
-def get_pandoc_formats_pre_1_18():
+def get_pandoc_formats_pre_1_18() -> Iterable:
     '''
     Dynamic preprocessor for Pandoc formats for version < 1.18.
     Return 2 lists. "from_formats" and "to_formats".
@@ -494,7 +572,7 @@ def get_pandoc_formats_pre_1_18():
 
 # copied and adapted from jupyter_nbconvert/utils/pandoc.py, Modified BSD License
 
-def _get_pandoc_version(pandoc_path):
+def _get_pandoc_version(pandoc_path:str) -> str:
     new_env = os.environ.copy()
     creation_flag = 0x08000000 if sys.platform == "win32" else 0 # set creation flag to not open pandoc in new console on windows
     if 'HOME' not in os.environ:
@@ -519,7 +597,7 @@ def _get_pandoc_version(pandoc_path):
     return version
 
 
-def get_pandoc_version():
+def get_pandoc_version() -> str:
     """Gets the Pandoc version if Pandoc is installed.
 
     It will probe Pandoc for its version, cache it and return that value. If a cached version is
@@ -537,7 +615,7 @@ def get_pandoc_version():
     return __version
 
 
-def get_pandoc_path():
+def get_pandoc_path() -> str:
     """Gets the Pandoc path if Pandoc is installed.
 
     It will return a path to pandoc which is used by pypandoc.
@@ -557,12 +635,12 @@ def get_pandoc_path():
     _ensure_pandoc_path()
     return __pandoc_path
 
-def ensure_pandoc_minimal_version(major, minor=0):
+def ensure_pandoc_minimal_version(major:int, minor:int=0) -> bool:
     """Check if the used pandoc fulfill a minimal version requirement.
 
-    :param bool major: pandoc major version, such as 1 or 2.
+    :param int major: pandoc major version, such as 1 or 2.
 
-    :param bool minor: pandoc minor version, such as 10 or 11.
+    :param int minor: pandoc minor version, such as 10 or 11.
 
     :returns: True if the installed pandoc is above the minimal version, False otherwise.
     :rtype: bool
@@ -574,12 +652,12 @@ def ensure_pandoc_minimal_version(major, minor=0):
     
 
 
-def ensure_pandoc_maximal_version(major, minor=9999):
+def ensure_pandoc_maximal_version(major:int, minor:int=9999) -> bool:
     """Check if the used pandoc fulfill a maximal version requirement.
 
-    :param bool major: pandoc major version, such as 1 or 2.
+    :param int major: pandoc major version, such as 1 or 2.
 
-    :param bool minor: pandoc minor version, such as 10 or 11.
+    :param int minor: pandoc minor version, such as 10 or 11.
 
     :returns: True if the installed pandoc is below the maximal version, False otherwise.
     :rtype: bool
@@ -590,7 +668,7 @@ def ensure_pandoc_maximal_version(major, minor=9999):
     return version[0] <= int(major) and version[1] <= int(minor)
 
 
-def _ensure_pandoc_path():
+def _ensure_pandoc_path() -> None:
     global __pandoc_path
     
     _check_log_handler()
@@ -639,10 +717,10 @@ def _ensure_pandoc_path():
             except Exception:
                 # we can't use that path...
                 if os.path.exists(path):
-                    # path exist but is not useable -> not executable?
+                    # path exist but is not usable -> not executable?
                     log_msg = ("Found {}, but not using it because of an "
                                "error:".format(path))
-                    logging.exception(log_msg)
+                    logger.exception(log_msg)
                 continue
             version = [int(x) for x in version_string.split(".")]
             while len(version) < len(curr_version):
@@ -687,11 +765,10 @@ def _ensure_pandoc_path():
                           "install pypandoc wheels with included pandoc.")
 
 
-def ensure_pandoc_installed(url=None, 
-                            targetfolder=None,
-                            version="latest",
-                            quiet=None,
-                            delete_installer=False):
+def ensure_pandoc_installed(url:Union[str, None]=None, 
+                            targetfolder:Union[str, None]=None,
+                            version:str="latest",
+                            delete_installer:bool=False) -> None:
     """Try to install pandoc if it isn't installed.
 
     Parameters are passed to download_pandoc()
@@ -699,11 +776,6 @@ def ensure_pandoc_installed(url=None,
     :raises OSError: if pandoc cannot be installed
     """
 
-    if quiet is not None:
-        msg = ("The quiet flag in PyPandoc has been deprecated in favour of "
-               "logging. See README.md for more information.")
-        warnings.warn(msg, DeprecationWarning, stacklevel=2)
-
     # Append targetfolder to the PATH environment variable so it is found by subprocesses
     if targetfolder is not None:
         os.environ["PATH"] = os.environ.get("PATH", "") + os.pathsep + os.path.abspath(os.path.expanduser(targetfolder))
